@@ -159,6 +159,10 @@ VILLA *vlopen(const char *name, int omode, VLCFUNC cmp){
     leaf = vlleafnew(villa, -1, -1);
     villa->root = leaf->id;
     villa->last = leaf->id;
+    if(!vltranbegin(villa) || !vltranabort(villa)){
+      vlclose(villa);
+      return NULL;
+    }
   }
   return villa;
 }
@@ -1370,7 +1374,10 @@ static VLLEAF *vlleafload(VILLA *villa, int id){
   ksiz = -1;
   prev = -1;
   next = -1;
-  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))) return NULL;
+  if(!(buf = dpget(villa->depot, (char *)&id, sizeof(int), 0, -1, &size))){
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
   if(_qdbm_inflate && villa->zmode){
     if(!(zbuf = _qdbm_inflate(buf, size, &zsiz))){
       dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
