@@ -40,6 +40,7 @@
 
 enum {                                   /* enumeration for actions */
   ACTVIEW,                               /* show articles */
+  ACTCAL,                                /* show calendars */
   ACTLIST,                               /* show list of articles */
   ACTSEARCH,                             /* show the search form and its result */
   ACTARTFORM,                            /* show the form to post an article */
@@ -93,6 +94,7 @@ const char *g_feedlabel = NULL;          /* label to get the feed the page or an
 const char *g_reloadlabel = NULL;        /* label to reload the page */
 const char *g_searchlabel = NULL;        /* label to search articles */
 const char *g_listlabel = NULL;          /* label to show list of all articles */
+const char *g_callabel = NULL;           /* label to show calendars */
 const char *g_prevlabel = NULL;          /* label to go to the previous page */
 const char *g_nextlabel = NULL;          /* label to go to the next page */
 const char *g_editlabel = NULL;          /* label to edit the existing */
@@ -133,6 +135,8 @@ const char *p_author = NULL;             /* author of an article */
 const char *p_text = NULL;               /* body text of an article */
 const char *p_password = NULL;           /* password of an article */
 const char *p_hash = NULL;               /* hash code of an article */
+int p_year = 1;                          /* creation year of articles to be shown */
+const char *p_cdate = NULL;              /* creation date of articles to be shown */
 const char *p_feed = NULL;               /* ID of an article for Atom feed */
 const char *c_author = NULL;             /* author in the cookie */
 const char *a_author = NULL;             /* author via basic authentication */
@@ -140,7 +144,10 @@ const char *a_author = NULL;             /* author via basic authentication */
 
 /* other global variables */
 RBBS *g_rbbs = NULL;                     /* database handler */
-const char *g_curdate = NULL;            /* current date */
+const char *g_curdate = NULL;            /* current date string */
+int g_curyear = 1;                       /* current year */
+int g_curmonth = 1;                      /* current month */
+int g_curday = 1;                        /* current day */
 int g_rmpost = FALSE;                    /* whether the request method is POST */
 int g_cannext = FALSE;                   /* whether the next page can be shown */
 const char *g_rtryres = "";              /* text for re-try to post a response */
@@ -166,6 +173,10 @@ void shownote(const char *msg);
 void showpagenavi(void);
 void showcaption(void);
 int matchingart(ARTICLE *art);
+void showcalendars(void);
+void showmonthcal(int year, int month);
+int dayofweek(int year, int month, int day);
+int daysofmonth(int year, int month);
 void showlist(void);
 void showsearchform(void);
 void showarticles(void);
@@ -274,6 +285,8 @@ int main(int argc, char **argv){
       g_searchlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "listlabel:")){
       g_listlabel = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "callabel:")){
+      g_callabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "prevlabel:")){
       g_prevlabel = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "nextlabel:")){
@@ -350,6 +363,7 @@ int main(int argc, char **argv){
   if(!g_reloadlabel) senderror("reloadlabel is undefined.");
   if(!g_searchlabel) senderror("searchlabel is undefined.");
   if(!g_listlabel) senderror("listlabel is undefined.");
+  if(!g_callabel) senderror("callabel is undefined.");
   if(!g_prevlabel) senderror("prevlabel is undefined.");
   if(!g_nextlabel) senderror("nextlabel is undefined.");
   if(!g_editlabel) senderror("editlabel is undefined.");
@@ -398,6 +412,8 @@ int main(int argc, char **argv){
   if((tmp = cbmapget(params, "text", -1, NULL)) != NULL) p_text = tmp;
   if((tmp = cbmapget(params, "password", -1, NULL)) != NULL) p_password = tmp;
   if((tmp = cbmapget(params, "hash", -1, NULL)) != NULL) p_hash = tmp;
+  if((tmp = cbmapget(params, "year", -1, NULL)) != NULL) p_year = atoi(tmp);
+  if((tmp = cbmapget(params, "cdate", -1, NULL)) != NULL) p_cdate = tmp;
   if((tmp = cbmapget(params, "feed", -1, NULL)) != NULL) p_feed = tmp;
   if(!p_userpass) p_userpass = "";
   if(p_action < ACTVIEW || p_action > ACTHELP) p_action = ACTVIEW;
@@ -412,6 +428,8 @@ int main(int argc, char **argv){
   if(!p_text) p_text = "";
   if(!p_password) p_password = "";
   if(!p_hash) p_hash = "";
+  if(p_year < 1 || p_year > 9999) p_year = 1;
+  if(!p_cdate) p_cdate = "";
   if(!p_feed) p_feed = "";
   /* read cookies */
   cooks = getcookies();
@@ -424,6 +442,7 @@ int main(int argc, char **argv){
   /* open the database */
   switch(p_action){
   case ACTVIEW:
+  case ACTCAL:
   case ACTLIST:
   case ACTSEARCH:
   case ACTARTFORM:
@@ -448,6 +467,9 @@ int main(int argc, char **argv){
   buf = datecurrent();
   cbglobalgc(buf, free);
   g_curdate = buf;
+  g_curyear = atoi(g_curdate);
+  g_curmonth = atoi(g_curdate + 5);
+  g_curday = atoi(g_curdate + 8);
   /* show page frame */
   if(p_action == ACTDUMPDB){
     senddump();
@@ -744,7 +766,8 @@ void sendhtml(void){
   } else {
     printf("Content-Disposition: inline; filename=%s\r\n", PAGENAME);
   }
-  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTLIST || p_action == ACTSEARCH)){
+  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTCAL || p_action == ACTLIST ||
+                p_action == ACTSEARCH)){
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
@@ -763,7 +786,8 @@ void sendhtml(void){
           g_pagelang, g_pagelang);
   xprintf("<head>\n");
   xprintf("<title>%@</title>\n", g_pagetitle);
-  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTLIST || p_action == ACTSEARCH)){
+  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTCAL || p_action == ACTLIST ||
+                p_action == ACTSEARCH)){
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
     xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
     free(tmp);
@@ -791,7 +815,7 @@ void sendhtml(void){
   }
   xprintf("</head>\n");
   xprintf("<body>\n");
-  if((p_action == ACTVIEW || p_action == ACTSEARCH) && p_id[0] == '\0'){
+  if((p_action == ACTVIEW || p_action == ACTSEARCH) && p_id[0] == '\0' && p_cdate[0] == '\0'){
     showcaption();
     showlist();
   } else if(p_action == ACTHELP){
@@ -804,6 +828,9 @@ void sendhtml(void){
     case ACTVIEW:
       showarticles();
       break;
+    case ACTCAL:
+      showcalendars();
+      break;
     case ACTLIST:
       showlist();
       break;
@@ -887,6 +914,7 @@ int authorize(void){
   artadm = FALSE;
   switch(p_action){
   case ACTVIEW:
+  case ACTCAL:
   case ACTLIST:
   case ACTSEARCH:
   case ACTHELP:
@@ -959,6 +987,7 @@ void showpagenavi(void){
   static int fst = TRUE;
   CBDATUM *buf;
   const char *fstr, *cond;
+  char idbuf[NUMBUFSIZ];
   buf = cbdatumopen("", 0);
   if(p_action == ACTSEARCH){
     xsprintf(buf, "&amp;action=%d", p_action);
@@ -991,13 +1020,24 @@ void showpagenavi(void){
           g_systemuri, ACTSEARCH, ++g_tabidx, g_searchlabel);
   xprintf("<a href=\"%@?action=%d\" rel=\"contents\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
           g_systemuri, ACTLIST, ++g_tabidx, g_listlabel);
-  if(p_page > 0){
+  sprintf(idbuf, "M%04d%02d", g_curyear, g_curmonth);
+  xprintf("<a href=\"%@?action=%d&amp;year=%d#%@\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+          g_systemuri, ACTCAL, g_curyear, idbuf, ++g_tabidx, g_callabel);
+  if(p_action == ACTCAL){
+    xprintf("<a href=\"%@?action=%d&amp;year=%d\" rel=\"next\""
+            " class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+            g_systemuri, ACTCAL, p_year - 1, ++g_tabidx, g_prevlabel);
+  } else if(p_page > 0){
     xprintf("<a href=\"%@?page=%d%s\" rel=\"prev\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, p_page - 1, cond, ++g_tabidx, g_prevlabel);
   } else {
     xprintf("<span class=\"navivoid\">%@</span>\n", g_prevlabel);
   }
-  if(g_cannext && p_action != ACTLIST){
+  if(p_action == ACTCAL){
+    xprintf("<a href=\"%@?action=%d&amp;year=%d\" rel=\"next\""
+            " class=\"navianc\" tabindex=\"%d\">%@</a>\n",
+            g_systemuri, ACTCAL, p_year + 1, ++g_tabidx, g_nextlabel);
+  } else if(g_cannext && p_action != ACTLIST){
     xprintf("<a href=\"%@?page=%d%s\" rel=\"next\" class=\"navianc\" tabindex=\"%d\">%@</a>\n",
             g_systemuri, p_page + 1, cond, ++g_tabidx, g_nextlabel);
   } else {
@@ -1055,6 +1095,118 @@ int matchingart(ARTICLE *art){
 }
 
 
+/* show calendars */
+void showcalendars(void){
+  ARTICLE *art;
+  CBLIST *list;
+  char *date, *pv;
+  int i, j, year, month, day;
+  if(!g_rbbs){
+    shownote("There is no corresponding article.");
+    return;
+  }
+  year = p_year;
+  month = 1;
+  xprintf("<div class=\"calenders\">\n");
+  xprintf("<table summary=\"calenders\" class=\"calenders\">\n");
+  for(i = 0; i < 12; i++){
+    xprintf("<tr>\n");
+    xprintf("<td class=\"cal\">\n");
+    showmonthcal(year, month);
+    xprintf("</td>\n");
+    xprintf("<td class=\"list\">\n");
+    for(day = 1; day <= 31; day++){
+      list = getcdatelist(g_rbbs, year, month, day);
+      for(j = 0; j < cblistnum(list); j++){
+        if(!(art = getarticle(g_rbbs, cblistval(list, j, NULL)))) continue;
+        date = cbmemdup(artcdate(art), -1);
+        if((pv = strchr(date, ' ')) != NULL) *pv = '\0';
+        xprintf("<div class=\"dlist\">\n");
+        xprintf("<span class=\"date\">%@</span>\n", date);
+        xprintf("<a href=\"%@?id=%@\" class=\"subject\">%@</a>\n",
+                g_systemuri, artid(art), artsubject(art));
+        xprintf("</div>\n");
+        free(date);
+        freearticle(art);
+      }
+      cblistclose(list);
+    }
+    xprintf("</td>\n");
+    xprintf("</tr>\n");
+    month++;
+    if(month > 12){
+      year++;
+      month = 1;
+    }
+  }
+  xprintf("</table>\n");
+  xprintf("</div>\n");
+}
+
+
+/* show the calendar of a month */
+void showmonthcal(int year, int month){
+  CBLIST *list;
+  char numbuf[NUMBUFSIZ], idbuf[NUMBUFSIZ];
+  int i, dm, dw, day;
+  dw = dayofweek(year, month, 1);
+  dm = daysofmonth(year, month);
+  day = 1;
+  xprintf("<div class=\"mcal\">\n");
+  sprintf(numbuf, "%04d-%02d", year, month);
+  sprintf(idbuf, "M%04d%02d", year, month);
+  xprintf("<div id=\"%@\" class=\"mcap\">%@</div>\n", idbuf, numbuf);
+  xprintf("<table summary=\"%@\">\n", numbuf);
+  xprintf("<tr>\n");
+  for(i = 0; i < 42; i++){
+    if(i > 0 && i % 7 == 0){
+      xprintf("</tr>\n");
+      xprintf("<tr>\n");
+    }
+    if(i >= dw && day <= dm){
+      xprintf("<td class=\"%@%@\">", i % 7 == 0 ? "sunday" : i % 7 == 6 ? "saturday" : "weekday",
+              year == g_curyear && month == g_curmonth && day == g_curday ? " today" : "");
+      list = getcdatelist(g_rbbs, year, month, day);
+      if(cblistnum(list) > 0){
+        sprintf(numbuf, "%04d-%02d-%02d", year, month, day);
+        xprintf("<a href=\"%@?sort=%d&amp;cdate=%@\">%d</a>",
+                g_systemuri, SORTCDATEA, numbuf, day);
+      } else {
+        xprintf("%d", day);
+      }
+      cblistclose(list);
+      xprintf("</td>\n");
+      day++;
+    } else {
+      xprintf("<td class=\"padding\">~</td>\n");
+    }
+  }
+  xprintf("</tr>\n");
+  xprintf("</table>\n");
+  xprintf("</div>\n");
+}
+
+
+/* get the day of week of a day */
+int dayofweek(int year, int month, int day){
+  return year -= month < 3,
+    (year + year / 4 - year / 100 + year / 400 + "032503514624"[month-1] - '0' + day) % 7;
+}
+
+
+/* get the number of days of a month */
+int daysofmonth(int year, int month){
+  switch(month){
+  case 2:
+    if(year % 400 == 0 || (year % 4 == 0 && year % 100 != 0)) return 29;
+    return 28;
+  case 4: case 6: case 9: case 11:
+    return 30;
+  }
+  return 31;
+}
+
+
 /* show titles of articles */
 void showlist(void){
   ARTICLE *art;
@@ -1198,9 +1350,10 @@ void showsearchform(void){
 
 /* show articles */
 void showarticles(void){
+  CBLIST *list;
   ARTICLE *art;
   char *id;
-  int skip, snum;
+  int i, skip, snum, year, month, day;
   snum = 0;
   if(g_rbbs && p_id[0] != '\0'){
     if((art = getarticle(g_rbbs, p_id)) != NULL){
@@ -1208,6 +1361,19 @@ void showarticles(void){
       snum++;
       freearticle(art);
     }
+  } else if(g_rbbs && p_cdate[0] != '\0'){
+    year = atoi(p_cdate);
+    month = strlen(p_cdate) > 5 ? atoi(p_cdate + 5) : 1;
+    day = strlen(p_cdate) > 8 ? atoi(p_cdate + 8) : 1;
+    list = getcdatelist(g_rbbs, year, month, day);
+    for(i = 0; i < cblistnum(list); i++){
+      if((art = getarticle(g_rbbs, cblistval(list, i, NULL))) != NULL){
+        showartwithctl(art);
+        snum++;
+        freearticle(art);
+      }
+    }
+    cblistclose(list);
   } else if(g_rbbs){
     setsortorder(g_rbbs, p_sort);
     skip = 0;
