@@ -206,6 +206,30 @@ char *getnextid(RBBS *rbbs){
 }
 
 
+/* Get the list of the ID strings of the articles created in a day. */
+CBLIST *getcdatelist(RBBS *rbbs, int year, int month, int day){
+  char date[NUMBUFSIZ], *key, *val;
+  CBLIST *list;
+  assert(rbbs);
+  list = cblistopen();
+  sprintf(date, "%04d-%02d-%02d", year, month, day);
+  vlcurjump(rbbs->cdidx, date, -1, VL_JFORWARD);
+  while((key = vlcurkey(rbbs->cdidx, NULL)) != NULL){
+    if(!cbstrfwmatch(key, date)){
+      free(key);
+      break;
+    }
+    if((val = vlcurval(rbbs->cdidx, NULL)) != NULL){
+      cblistpush(list, val, -1);
+      free(val);
+    }
+    free(key);
+    vlcurnext(rbbs->cdidx);
+  }
+  return list;
+}
+
+
 /* Retrieve article from a database. */
 ARTICLE *getarticle(RBBS *rbbs, const char *id){
   ARTICLE *art;
